#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import math

from django import forms
from django.core import exceptions, validators
from django.db import models
from django.db.models.fields import IntegerField
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _
from dj.choices import unset, Choices, Gender


class ChoiceField(IntegerField):
    description = _("Integer")
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if kwargs.get('_in_south'): # workaround for South removing `choices`
            kwargs['choices'] = Gender
            del kwargs['_in_south']
        if 'choices' not in kwargs:
            raise exceptions.ImproperlyConfigured("No choices class specified.")
        else:
            try:
                if not issubclass(kwargs['choices'], Choices):
                    raise TypeError()
            except TypeError:
                raise exceptions.ImproperlyConfigured("dj.choices class "
                        "required as `choices` argument.")
        self.choice_class = kwargs['choices']
        self.item_getter = kwargs.get('item', lambda x: (x.id,))
        kwargs['choices'] = self.choice_class(item=kwargs.get('item', unset),
            filter=kwargs.get('filter', (unset,)), grouped=kwargs.get('grouped',
                False))
        if isinstance(kwargs.get('default'), Choices.Choice):
            kwargs['default'] = self.item_getter(kwargs['default'])[0]
        for arg in 'filter', 'grouped', 'item':
            if arg in kwargs:
                del kwargs[arg]
        super(ChoiceField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "IntegerField"

    def to_python(self, value):
        value = super(ChoiceField, self).to_python(self.get_prep_value(value))
        if value is None:
            return value
        try:
            return self.choice_class.from_id(value)
        except (TypeError, ValueError):
            raise exceptions.ValidationError(
                self.error_messages['invalid_choice'] % value)

    def from_python(self, value):
        return self.get_prep_value(self.to_python(value))

    def get_prep_value(self, value):
        if value in validators.EMPTY_VALUES:
            return None
        if isinstance(value, unicode):
            return int(value)
        if isinstance(value, (int, long)):
            return value
        return self.item_getter(value)[0]

    def get_prep_lookup(self, lookup_type, value):
        if lookup_type in ('exact', 'lt', 'lte', 'gt', 'gte'):
            if lookup_type in ('gte', 'lt') and isinstance(value, float):
                value = math.ceil(value)
            else:
                value = self.get_prep_value(value)
        elif lookup_type in ('in', 'range'):
            value = [self.get_prep_value(v) for v in value]
        elif lookup_type != 'isnull':
            raise TypeError('Invalid lookup_type: %r' % lookup_type)
        return super(ChoiceField, self).get_prep_lookup(lookup_type, value)

    def validate(self, value, model_instance):
        return super(ChoiceField, self).validate(self.get_prep_value(value),
                model_instance)

    def formfield(self, form_class=forms.CharField, **kwargs):
        """Has to be defined as a whole without doing super() because of
        Django bug #9245."""
        defaults = {'required': not self.blank, 'label': capfirst(self.verbose_name), 'help_text': self.help_text}
        if self.has_default():
            if callable(self.default):
                defaults['initial'] = self.default
                defaults['show_hidden_initial'] = True
            else:
                defaults['initial'] = self.get_default()
        if self.choices:
            # Fields with choices get special treatment.
            include_blank = self.blank or not (self.has_default() or 'initial' in kwargs)
            defaults['choices'] = self.get_choices(include_blank=include_blank)
            defaults['coerce'] = self.from_python # XXX: changed
            if self.null:
                defaults['empty_value'] = None
            form_class = _TypedChoiceField # XXX: changed
            # Many of the subclass-specific formfield arguments (min_value,
            # max_value) don't apply for choice fields, so be sure to only pass
            # the values that TypedChoiceField will understand.
            for k in kwargs.keys():
                if k not in ('coerce', 'empty_value', 'choices', 'required',
                             'widget', 'label', 'initial', 'help_text',
                             'error_messages', 'show_hidden_initial'):
                    del kwargs[k]
        defaults.update(kwargs)
        return form_class(**defaults)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)

    def south_field_triple(self):
        kwargs = dict(
            null=repr(self.null),
            blank=repr(self.blank),
            db_column=repr(self.db_column),
            db_index=repr(self.db_index),
            primary_key=repr(self.primary_key),
            unique=repr(self.unique),
            _in_south=repr(True),
        )
        if self.default is not models.NOT_PROVIDED:
            kwargs['default'] = repr(self.default)
        return ('dj.choices.fields.ChoiceField', [], kwargs)


class _TypedChoiceField(forms.TypedChoiceField):
    def clean(self, value):
        return self.coerce(value)

    def prepare_value(self, value):
        return self.coerce(value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.db import models
from dj.choices import Choices
from dj.choices.fields import ChoiceField


class Color(Choices):
    _ = Choices.Choice

    red = _("Red")
    green = _("Green")
    blue = _("Blue")

class MusicGenre(Choices):
    _ = Choices.Choice

    banjo = _("Banjo")
    country = _("Country")
    pop = _("Pop")
    rap = _("Rap")
    rock = _("Rock")

class Sports(Choices):
    _ = Choices.Choice

    TEAM_SPORTS = Choices.Group(0, 'Team Sports')
    soccer = _("Soccer")
    basketball = _("Basketball")
    baseball = _("Baseball")

    INDIVIDUAL = Choices.Group(100, 'Individual Sports')
    swimming = _("Swimming")
    mountaineering = _("Mountaineering")
    bicycling = _("Bicycling")

    NOT_REALLY = Choices.Group(200, 'Fake Sports')
    chess = _("Chess")
    bridge = _("Bridge")
    poker = _("Poker")


class Favourites(models.Model):
    name = models.CharField(max_length=100)
    color = ChoiceField(choices=Color, default=Color.green)
    music = ChoiceField(choices=MusicGenre, filter=('rock', 'country',
        'banjo'))
    sport = ChoiceField(choices=Sports, grouped=True)
    nullable = ChoiceField(choices=Color, default=None, null=True, blank=True)

    @classmethod
    def create(cls, **kwargs):
        defaults = {
            'music': MusicGenre.banjo,
            'sport': Sports.poker,
            'color': Color.green,
            'nullable': None,
        }
        defaults.update(kwargs)
        return cls.objects.create(**defaults)


class RegularIntegers(models.Model):
    """A helper model to help test behaviour of the regular IntegerFields."""
    color = models.IntegerField(choices=Color(), default=Color.green.id)

    @classmethod
    def create(cls, **kwargs):
        defaults = {
            'color': Color.green.id,
        }
        defaults.update(kwargs)
        return cls.objects.create(**defaults)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.conf import settings
from django.test import TestCase
from django.test.simple import DjangoTestSuiteRunner, reorder_suite
from django.utils.importlib import import_module
from django.utils.unittest.loader import defaultTestLoader


class DiscoveryDjangoTestSuiteRunner(DjangoTestSuiteRunner):
    """A test suite runner that uses unittest2 test discovery.
    Courtesy of @carljm."""

    def build_suite(self, test_labels, extra_tests=None, **kwargs):
        suite = None
        discovery_root = settings.TEST_DISCOVERY_ROOT

        if test_labels:
            suite = defaultTestLoader.loadTestsFromNames(test_labels)
            # if single named module has no tests, do discovery within it
            if not suite.countTestCases() and len(test_labels) == 1:
                suite = None
                discovery_root = import_module(test_labels[0]).__path__[0]

        if suite is None:
            suite = defaultTestLoader.discover(
                discovery_root,
                top_level_dir=settings.BASE_PATH,
                )

        if extra_tests:
            for test in extra_tests:
                suite.addTest(test)

        return reorder_suite(suite, (TestCase,))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Tests for choices."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


from django.test import TestCase


class SimpleTest(TestCase):
    def test_dummy(self):
        """Just see if the import works as expected."""
        from dj import choices

    def test_choices_basic(self):
        from dj.choices import Choices

        class Colors(Choices):
            _ = Choices.Choice

            white = _("White")
            yellow = _("Yellow")
            red = _("Red")
            green = _("Green")
            black = _("Black")

        self.assertEqual(Colors(), [(1, "White"), (2, "Yellow"), (3, "Red"),
                (4, "Green"), (5, "Black")])
        self.assertEqual(Colors.white.id, 1)
        self.assertEqual(Colors.white.desc, "White")
        self.assertEqual(Colors.white.name, "white")
        self.assertEqual(Colors.from_id(1), Colors.white)
        self.assertEqual(Colors.name_from_id(1), Colors.white.name)
        self.assertEqual(Colors.desc_from_id(1), Colors.white.desc)
        self.assertEqual(Colors.raw_from_id(1), Colors.white.raw)
        self.assertEqual(Colors.from_name("white"), Colors.white)
        self.assertEqual(Colors.id_from_name("white"), Colors.white.id)
        self.assertEqual(Colors.desc_from_name("white"), Colors.white.desc)
        self.assertEqual(Colors.raw_from_name("white"), Colors.white.raw)
        self.assertEqual(Colors.yellow.id, 2)
        self.assertEqual(Colors.yellow.desc, "Yellow")
        self.assertEqual(Colors.yellow.name, "yellow")
        self.assertEqual(Colors.from_id(2), Colors.yellow)
        self.assertEqual(Colors.name_from_id(2), Colors.yellow.name)
        self.assertEqual(Colors.desc_from_id(2), Colors.yellow.desc)
        self.assertEqual(Colors.raw_from_id(2), Colors.yellow.raw)
        self.assertEqual(Colors.from_name("yellow"), Colors.yellow)
        self.assertEqual(Colors.id_from_name("yellow"), Colors.yellow.id)
        self.assertEqual(Colors.desc_from_name("yellow"), Colors.yellow.desc)
        self.assertEqual(Colors.raw_from_name("yellow"), Colors.yellow.raw)
        self.assertEqual(Colors.red.id, 3)
        self.assertEqual(Colors.red.desc, "Red")
        self.assertEqual(Colors.red.name, "red")
        self.assertEqual(Colors.from_id(3), Colors.red)
        self.assertEqual(Colors.name_from_id(3), Colors.red.name)
        self.assertEqual(Colors.desc_from_id(3), Colors.red.desc)
        self.assertEqual(Colors.raw_from_id(3), Colors.red.raw)
        self.assertEqual(Colors.from_name("red"), Colors.red)
        self.assertEqual(Colors.id_from_name("red"), Colors.red.id)
        self.assertEqual(Colors.desc_from_name("red"), Colors.red.desc)
        self.assertEqual(Colors.raw_from_name("red"), Colors.red.raw)
        self.assertEqual(Colors.green.id, 4)
        self.assertEqual(Colors.green.desc, "Green")
        self.assertEqual(Colors.green.name, "green")
        self.assertEqual(Colors.from_id(4), Colors.green)
        self.assertEqual(Colors.name_from_id(4), Colors.green.name)
        self.assertEqual(Colors.desc_from_id(4), Colors.green.desc)
        self.assertEqual(Colors.raw_from_id(4), Colors.green.raw)
        self.assertEqual(Colors.from_name("green"), Colors.green)
        self.assertEqual(Colors.id_from_name("green"), Colors.green.id)
        self.assertEqual(Colors.desc_from_name("green"), Colors.green.desc)
        self.assertEqual(Colors.raw_from_name("green"), Colors.green.raw)
        self.assertEqual(Colors.black.id, 5)
        self.assertEqual(Colors.black.desc, "Black")
        self.assertEqual(Colors.black.name, "black")
        self.assertEqual(Colors.from_id(5), Colors.black)
        self.assertEqual(Colors.name_from_id(5), Colors.black.name)
        self.assertEqual(Colors.desc_from_id(5), Colors.black.desc)
        self.assertEqual(Colors.raw_from_id(5), Colors.black.raw)
        self.assertEqual(Colors.from_name("black"), Colors.black)
        self.assertEqual(Colors.id_from_name("black"), Colors.black.id)
        self.assertEqual(Colors.desc_from_name("black"), Colors.black.desc)
        self.assertEqual(Colors.raw_from_name("black"), Colors.black.raw)

    def test_choices_groups(self):
        from dj.choices import Choices

        class Groupies(Choices):
            _ = Choices.Choice

            GROUP1 = Choices.Group(0)
            entry1 = _("entry1")
            entry2 = _("entry2")
            entry3 = _("entry3")

            GROUP2 = Choices.Group(10)
            entry4 = _("entry4")
            entry5 = _("entry5")
            entry6 = _("entry6")

            GROUP3 = Choices.Group(20)
            entry7 = _("entry7")
            entry8 = _("entry8")
            entry9 = _("entry9")

        self.assertEqual(Groupies(), [(1, u'entry1'), (2, u'entry2'),
                (3, u'entry3'), (11, u'entry4'), (12, u'entry5'),
                (13, u'entry6'), (21, u'entry7'), (22, u'entry8'),
                (23, u'entry9')])
        self.assertEqual(Groupies.entry1.group, Groupies.GROUP1)
        self.assertEqual(Groupies.entry2.group, Groupies.GROUP1)
        self.assertEqual(Groupies.entry3.group, Groupies.GROUP1)
        self.assertEqual(Groupies.entry4.group, Groupies.GROUP2)
        self.assertEqual(Groupies.entry5.group, Groupies.GROUP2)
        self.assertEqual(Groupies.entry6.group, Groupies.GROUP2)
        self.assertEqual(Groupies.entry7.group, Groupies.GROUP3)
        self.assertEqual(Groupies.entry8.group, Groupies.GROUP3)
        self.assertEqual(Groupies.entry9.group, Groupies.GROUP3)
        self.assertEqual(Groupies.GROUP1.choices, [Groupies.entry1,
                Groupies.entry2, Groupies.entry3])
        self.assertEqual(Groupies.GROUP2.choices, [Groupies.entry4,
                Groupies.entry5, Groupies.entry6])
        self.assertEqual(Groupies.GROUP3.choices, [Groupies.entry7,
                Groupies.entry8, Groupies.entry9])

    def test_choices_validation(self):
        from dj.choices import Choices

        class NoChoice(Choices):
            not_a_choice = "Not a Choice() object"

        try:
            NoChoice()
            self.fail("ValueError not raised.")
        except ValueError, e:
            self.assertEqual(str(e), "Choices class declared with no actual "
                    "choice fields.")

    def test_choices_filter(self):
        from dj.choices import Country

        self.assertEqual(len(Country()), 235)
        self.assertEqual(Country(filter=("pl", "gb", "de")), [(73, u'Germany'),
            (153, u'Poland'), (202, u'United Kingdom')])

    def test_shifted_basic(self):
        from dj.choices import Choices

        class InvitationStatus(Choices):
            _ = Choices.Choice

            blocked = _("Blocked").extra(first_letter='b')
            rejected = _("Rejected").extra(first_letter='r')
            pending = _("Pending").extra(first_letter='p')
            accepted = _("Accepted").extra(first_letter='a')

        self.assertEqual(InvitationStatus.blocked.first_letter, 'b')
        self.assertEqual(InvitationStatus.rejected.first_letter, 'r')
        self.assertEqual(InvitationStatus.pending.first_letter, 'p')
        self.assertEqual(InvitationStatus.accepted.first_letter, 'a')

    def test_shifted_group_inheritance(self):
        from dj.choices import Choices

        class Colors(Choices):
            _ = Choices.Choice

            NICE = Choices.Group(0).extra(comment='Nice!')
            blue = _("Blue")
            green = _("Green").extra(comment='This is it.')
            brown = _("Brown")

            UGLY = Choices.Group(10).extra(comment='Ugly!')
            pink = _("Pink")
            red = _("Red")
            toxic_waste_green = _("Toxic waste green").extra(comment='Yuk!')

        self.assertEqual(Colors.blue.comment, 'Nice!')
        self.assertEqual(Colors.green.comment, 'This is it.')
        self.assertEqual(Colors.brown.comment, 'Nice!')
        self.assertEqual(Colors.pink.comment, 'Ugly!')
        self.assertEqual(Colors.red.comment, 'Ugly!')
        self.assertEqual(Colors.toxic_waste_green.comment, 'Yuk!')

    def test_choicefield(self):
        from dj._choicestestproject.app.models import Favourites, Color,\
                MusicGenre, Sports
        judy = Favourites.create(name='Judy')
        self.assertEqual(judy.color, Color.green)
        self.assertEqual(judy.music, MusicGenre.banjo)
        self.assertEqual(judy.sport, Sports.poker)
        self.assertEqual(judy.nullable, None)
        judy.color = Color.blue
        judy.music = MusicGenre.rock
        judy.sport = Sports.mountaineering
        judy.save()
        judy2 = Favourites.objects.get(name='Judy')
        self.assertEqual(judy2.color, Color.blue)
        self.assertEqual(judy2.music, MusicGenre.rock)
        self.assertEqual(judy2.sport, Sports.mountaineering)
        tom = Favourites.create(name='Tom', sport=Sports.mountaineering)
        self.assertEqual(Favourites.objects.filter(sport=102).count(), 2)
        self.assertEqual(Favourites.objects.filter(
            sport=Sports.mountaineering).count(), 2)
        self.assertEqual(Favourites.objects.filter(
            sport__gt=10).count(), 2)
        self.assertEqual(Favourites.objects.filter(
            color__lte=2).count(), 1)
        tom.color = 1
        tom.save()
        tom2 = Favourites.objects.get(name='Tom')
        self.assertEqual(tom2.color, Color.red)
        tom_and_judy = Favourites.objects.filter(color__in=(Color.blue, Color.red))
        self.assertEqual(tom_and_judy.count(), 2)
        self.assertEqual({obj.name for obj in tom_and_judy}, {'Judy', 'Tom'})

    def test_form_with_choicefields(self):
        from dj._choicestestproject.app.models import Favourites, Color,\
                MusicGenre, Sports
        from dj._choicestestproject.app.forms import FavouritesForm
        empty_form = FavouritesForm()
        self.assertFalse(empty_form.is_valid())
        self.assertEqual(empty_form._bound_value('color'), Color.green.id)
        self.assertEqual(empty_form._bound_value('music'), None)
        self.assertEqual(empty_form._bound_value('sport'), None)
        self.assertEqual(empty_form._bound_value('name'), None)
        self.assertEqual(empty_form._bound_value('nullable'), None)
        empty_form_data = FavouritesForm(data={
                'color': 1, 'music': 2, 'sport': 3, 'name': 'Richard'})
        self.assertTrue(empty_form_data.is_valid())
        self.assertEqual(empty_form_data._bound_value('color'), Color.red.id)
        self.assertEqual(empty_form_data._bound_value('music'),
                MusicGenre.country.id)
        self.assertEqual(empty_form_data._bound_value('sport'), Sports.baseball.id)
        self.assertEqual(empty_form_data._bound_value('name'), 'Richard')
        self.assertEqual(empty_form_data._bound_value('nullable'), None)
        judy = Favourites.create(name='Judy')
        judy_form = FavouritesForm(instance=judy)
        self.assertFalse(judy_form.is_valid()) # because it's not bound
        self.assertEqual(judy_form._bound_value('color'), Color.green.id)
        self.assertEqual(judy_form._bound_value('music'), MusicGenre.banjo.id)
        self.assertEqual(judy_form._bound_value('sport'), Sports.poker.id)
        self.assertEqual(judy_form._bound_value('name'), 'Judy')
        self.assertEqual(judy_form._bound_value('nullable'), None)
        data_for_form = dict(judy_form.initial)
        data_for_form.update({'nullable': Color.red})
        judy_form_data = FavouritesForm(instance=judy, data=data_for_form)
        self.assertTrue(judy_form_data.is_valid())
        self.assertEqual(judy_form_data._bound_value('color'), Color.green.id)
        self.assertEqual(judy_form_data._bound_value('music'), MusicGenre.banjo.id)
        self.assertEqual(judy_form_data._bound_value('sport'), Sports.poker.id)
        self.assertEqual(judy_form_data._bound_value('name'), 'Judy')
        self.assertEqual(judy_form_data._bound_value('nullable'), Color.red.id)
        invalid_data_for_form = dict(judy_form.initial)
        invalid_data_for_form.update({'color': -1, 'music': '', 'sport': None,
                'nullable': ''})
        judy_form_invalid_data = FavouritesForm(instance=judy,
                data=invalid_data_for_form)
        self.assertFalse(judy_form_invalid_data.is_valid())
        self.assertEqual(set(judy_form_invalid_data.errors.keys()), {'color',
            'music', 'sport'})
        self.assertTrue('not a valid choice' in
                judy_form_invalid_data.errors['color'][0])
        self.assertTrue('cannot be null' in
                judy_form_invalid_data.errors['music'][0])
        self.assertTrue('cannot be null' in
                judy_form_invalid_data.errors['sport'][0])

    def test_regularintegers(self):
        from dj._choicestestproject.app.models import Color, RegularIntegers
        from dj._choicestestproject.app.forms import RegularIntegersForm
        rint = RegularIntegers.create()
        self.assertEqual(rint.color, Color.green.id)
        rint_invalid = RegularIntegers.create(color=-1)
        self.assertEqual(rint_invalid.color, -1)
        empty_form = RegularIntegersForm()
        self.assertFalse(empty_form.is_valid()) # because it's not bound
        valid_form = RegularIntegersForm(data={'color': 1})
        self.assertTrue(valid_form.is_valid())
        invalid_form = RegularIntegersForm(data={'color': -1})
        self.assertFalse(invalid_form.is_valid())
        self.assertTrue('color' in invalid_form.errors)
        self.assertTrue('Select a valid choice' in
                invalid_form.errors['color'][0])
        invalid_type_form = RegularIntegersForm(data={'color': 'aaa'})
        self.assertFalse(invalid_type_form.is_valid())
        self.assertTrue('color' in invalid_type_form.errors)
        self.assertTrue('Select a valid choice' in
                invalid_type_form.errors['color'][0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 - 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Choices - an enum implementation for Django forms and models."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from functools import partial
from textwrap import dedent


unset = object()
ugettext = unset


class ChoicesEntry(object):
    global_id = 0

    def __init__(self, description, id, name=None):
        self.id = id
        self.raw = description
        self.global_id = Choice.global_id
        self.name = name
        self.__extra__ = []
        ChoicesEntry.global_id += 1

    @property
    def desc(self):
        if not self.raw:
            return self.raw
        # ugettext obscured that way so you can use choices in settings.py
        global ugettext
        if ugettext is unset:
            from django.utils.translation import ugettext
        return ugettext(self.raw)

    def __unicode__(self, raw=False):
        name = self.name
        if raw:
            name = "{!r}".format(name)
            if name[0] in 'bru':
                name = name[2:-1]
            else:
                name = name[1:-1]
        return "<{}: {} (id: {})>".format(self.__class__.__name__,
            name, self.id)

    def __repr__(self):
        return self.__unicode__(raw=True)

    def extra(self, **other):
        """Enables adding custom attributes to choices at declaration time.
        For example::

            class Color(Choices):
                _ = Choices.Choice

                red = _("red").extra(html='#ff0000')
                green = _("green").extra(html='#00ff00')
                blue = _("blue").extra(html='#0000ff')

        Later on you can use the defined attribute directly::

            >>> Color.red.html
            '#ff0000'
    
        or with choices received using the getters::

            >>> Color.from_name(request.POST['color']).html
            '#00ff00'
        """
        for key, value in other.iteritems():
            self.__extra__.append(key)
            setattr(self, key, value)
        return self

    def __lshift__(self, other):
        """Deprecated: use choice.extra(arg=value)."""
        return self.extra(**other)


class ChoiceGroup(ChoicesEntry):
    """A group of choices."""

    def __init__(self, id, description=''):
        super(ChoiceGroup, self).__init__(description, id=id)
        self.choices = []


class Choice(ChoicesEntry):
    """A single choice."""

    def __init__(self, description):
        super(Choice, self).__init__(description, id=-255)
        self.group = None

    def __unicode__(self, raw=False):
        name = self.name
        rawval = self.raw
        if raw: # for __repr__
            name = "{!r}".format(name)
            if name[0] in 'bru':
                name = name[2:-1]
            else:
                name = name[1:-1]
            rawval = "{!r}".format(rawval)
            if rawval[0] in 'bru':
                rawval = rawval[2:-1]
            else:
                rawval = rawval[1:-1]
        return "<{}: {} (id: {}, name: {})>".format(self.__class__.__name__,
            rawval, self.id, name)


def _getter(name, given, returns, found, getter):
    def impl(cls, id, found=lambda id, k, v: False,
                        getter=lambda id, k, v: None, fallback=unset):
        """Unless `fallback` is set, raises ValueError if name not present."""
        for k, v in cls.__dict__.items():
            if isinstance(v, ChoicesEntry) and found(id, k, v):
                return getter(id, k, v)
        if fallback is unset:
            raise ValueError("Nothing found for '{}'.".format(id))
        else:
            return fallback
    function = partial(impl, found=found, getter=getter)
    function.__name__ = name
    function.__doc__ = ("Choices.{name}({given}, fallback=unset) -> {returns}"
        "\n\nGiven the `{given}`, returns the `{returns}`. {impl_doc}"
        "".format(name=name, given=given, returns=returns,
            impl_doc=impl.__doc__))
    return classmethod(function)


class _ChoicesMeta(type):
    def __new__(meta, classname, bases, classDict):
        groups = []
        values = []
        for k, v in classDict.items():
            if isinstance(v, ChoicesEntry):
                v.name = k.strip('_')
                values.append(v)
        values.sort(lambda x, y: x.global_id - y.global_id)
        last_choice_id = 0
        group = None
        for choice in values:
            if isinstance(choice, ChoiceGroup):
                last_choice_id = choice.id
                group = choice
                groups.append(group)
            else:
                if group:
                    group.choices.append(choice)
                    choice.group = group
                    for extra in group.__extra__:
                        if not hasattr(choice, extra):
                            setattr(choice, extra, getattr(group, extra))
                if choice.id == -255:
                    last_choice_id += 1
                    choice.id = last_choice_id
                last_choice_id = choice.id
        classDict['__groups__'] = groups
        classDict['__choices__'] = values
        return type.__new__(meta, classname, bases, classDict)


class Choices(list):
    __metaclass__ = _ChoicesMeta

    def __init__(self, filter=(unset,), item=unset, grouped=False):
        """Creates a list of pairs from the specified Choices class.
        By default, each pair consists of a numeric ID and the translated
        description. If `use_ids` is False, the name of the attribute
        is used instead of the numeric ID.

        If `filter` is specified, it's a set or sequence of attribute
        names that should be included in the list. Note that the numeric
        IDs are the same regardless of the filtering. This is useful
        for predefining a large set of possible values and filtering to
        only the ones which are currently implemented."""
        if not self.__choices__:
            raise ValueError("Choices class declared with no actual "
                             "choice fields.")
        if item is unset:
            item = lambda choice: (choice.id, choice.desc)
        filter = set(filter)
        if grouped and self.__groups__:
            for group in self.__groups__:
                group_choices = []
                for choice in group.choices:
                    if choice.name in filter or (unset in filter and
                        isinstance(choice, Choice)):
                        group_choices.append(item(choice))
                if group_choices:
                    self.append((group.desc, tuple(group_choices)))
        else:
            if grouped:
                import warnings
                warnings.warn("Choices class called with grouped=True and no "
                    "actual groups.")
            for choice in self.__choices__:
                if choice.name in filter or (unset in filter and
                    isinstance(choice, Choice)):
                    self.append(item(choice))

    from_name = _getter("from_name",
        given="name",
        returns="choice object",
        found=lambda id, k, v: k == id,
        getter=lambda id, k, v: v)

    id_from_name = _getter("id_from_name",
        given="name",
        returns="id",
        found=lambda id, k, v: k == id,
        getter=lambda id, k, v: v.id)

    desc_from_name = _getter("desc_from_name",
        given="name",
        returns="localized description string",
        found=lambda id, k, v: k == id,
        getter=lambda id, k, v: v.desc)

    raw_from_name = _getter("raw_from_name",
        given="name",
        returns="raw description string",
        found=lambda id, k, v: k == id,
        getter=lambda id, k, v: v.raw)

    from_id = _getter("from_id",
        given="id",
        returns="choice object",
        found=lambda id, k, v: v.id == id,
        getter=lambda id, k, v: v)

    name_from_id = _getter("name_from_id",
        given="id",
        returns="attribute name",
        found=lambda id, k, v: v.id == id,
        getter=lambda id, k, v: k)

    desc_from_id = _getter("desc_from_id",
        given="id",
        returns="localized description string",
        found=lambda id, k, v: v.id == id,
        getter=lambda id, k, v: v.desc)

    raw_from_id = _getter("raw_from_id",
        given="id",
        returns="raw description string",
        found=lambda id, k, v: v.id == id,
        getter=lambda id, k, v: v.raw)

    @staticmethod
    def to_ids(func):
        """Converts a sequence of choices to a sequence of choice IDs."""
        def wrapper(self):
            return (elem.id for elem in func(self))
        return wrapper

    @staticmethod
    def to_names(func):
        """Converts a sequence of choices to a sequence of choice names."""
        def wrapper(self):
            return (elem.name for elem in func(self))
        return wrapper

    Choice = Choice
    Group = ChoiceGroup

    # deprecated compatibility layer for code using lck.django.choices < 0.8
    # to be removed at 1.0
    FromName = from_name
    IDFromName = id_from_name
    DescFromName = desc_from_name
    RawFromName = raw_from_name
    FromID = from_id
    NameFromID = name_from_id
    DescFromID = desc_from_id
    RawFromID = raw_from_id
    ToIDs = to_ids
    ToNames = to_names


#
# commonly used choices
#

class Country(Choices):
    """Specifies a set for all countries of the world (as of January 2011),
    including unions, parts of United Kingdom and unrecognized states."""
    _ = Choices.Choice

    COUNTRIES = Choices.Group(0)
    """Officially recognized states, as of January 2011."""

    af = _("Afghanistan")
    al = _("Albania")
    dz = _("Algeria")
    as_ = _("American Samoa")
    ad = _("Andorra")
    ao = _("Angola")
    ai = _("Anguilla")
    aq = _("Antarctica")
    ag = _("Antigua and Barbuda")
    ar = _("Argentina")
    am = _("Armenia")
    aw = _("Aruba")
    au = _("Australia")
    at = _("Austria")
    az = _("Azerbaijan")
    bs = _("Bahamas")
    bh = _("Bahrain")
    bd = _("Bangladesh")
    bb = _("Barbados")
    by = _("Belarus")
    be = _("Belgium")
    bz = _("Belize")
    bj = _("Benin")
    bm = _("Bermuda")
    bt = _("Bhutan")
    bo = _("Bolivia")
    ba = _("Bosnia and Herzegovina")
    bw = _("Botswana")
    br = _("Brazil")
    bn = _("Brunei")
    bg = _("Bulgaria")
    bf = _("Burkina Faso")
    bi = _("Burundi")
    kh = _("Cambodia")
    cm = _("Cameroon")
    ca = _("Canada")
    cv = _("Cape Verde")
    ky = _("Cayman Islands")
    cf = _("Central African Republic")
    td = _("Chad")
    cl = _("Chile")
    cn = _("China")
    co = _("Colombia")
    km = _("Comoros")
    cg = _("Congo Brazzaville")
    cd = _("Congo Kinshasa")
    ck = _("Cook Islands")
    cr = _("Costa Rica")
    ci = _("Cote Divoire")
    hr = _("Croatia")
    cu = _("Cuba")
    cy = _("Cyprus")
    cz = _("Czech Republic")
    dk = _("Denmark")
    dj = _("Djibouti")
    dm = _("Dominica")
    do = _("Dominican Republic")
    ec = _("Ecuador")
    eg = _("Egypt")
    sv = _("El Salvador")
    gq = _("Equatorial Guinea")
    er = _("Eritrea")
    ee = _("Estonia")
    et = _("Ethiopia")
    fo = _("Faroe Islands")
    fj = _("Fiji")
    fi = _("Finland")
    fr = _("France")
    pf = _("French Polynesia")
    ga = _("Gabon")
    gm = _("Gambia")
    ge = _("Georgia")
    de = _("Germany")
    gh = _("Ghana")
    gi = _("Gibraltar")
    gr = _("Greece")
    gd = _("Grenada")
    gu = _("Guam")
    gt = _("Guatemala")
    gw = _("Guinea Bissau")
    gn = _("Guinea")
    gy = _("Guyana")
    ht = _("Haiti")
    hn = _("Honduras")
    hk = _("Hong Kong")
    hu = _("Hungary")
    is_ = _("Iceland")
    in_ = _("India")
    id = _("Indonesia")
    ir = _("Iran")
    iq = _("Iraq")
    ie = _("Ireland")
    il = _("Israel")
    it = _("Italy")
    jm = _("Jamaica")
    jp = _("Japan")
    je = _("Jersey")
    jo = _("Jordan")
    kz = _("Kazakhstan")
    ke = _("Kenya")
    ki = _("Kiribati")
    kw = _("Kuwait")
    kg = _("Kyrgyzstan")
    la = _("Laos")
    lv = _("Latvia")
    lb = _("Lebanon")
    ls = _("Lesotho")
    lr = _("Liberia")
    ly = _("Libya")
    li = _("Liechtenstein")
    lt = _("Lithuania")
    lu = _("Luxembourg")
    mo = _("Macau")
    mk = _("Macedonia")
    mg = _("Madagascar")
    mw = _("Malawi")
    my = _("Malaysia")
    mv = _("Maldives")
    ml = _("Mali")
    mt = _("Malta")
    mh = _("Marshall Islands")
    mr = _("Mauritania")
    mu = _("Mauritius")
    mx = _("Mexico")
    fm = _("Micronesia")
    md = _("Moldova")
    mc = _("Monaco")
    mn = _("Mongolia")
    me = _("Montenegro")
    ms = _("Montserrat")
    ma = _("Morocco")
    mz = _("Mozambique")
    mm = _("Myanmar")
    na = _("Namibia")
    nr = _("Nauru")
    np = _("Nepal")
    an = _("Netherlands Antilles")
    nl = _("Netherlands")
    nz = _("New Zealand")
    ni = _("Nicaragua")
    ne = _("Niger")
    ng = _("Nigeria")
    kp = _("North Korea")
    no = _("Norway")
    om = _("Oman")
    pk = _("Pakistan")
    pw = _("Palau")
    pa = _("Panama")
    pg = _("Papua New Guinea")
    py = _("Paraguay")
    pe = _("Peru")
    ph = _("Philippines")
    pl = _("Poland")
    pt = _("Portugal")
    pr = _("Puerto Rico")
    qa = _("Qatar")
    ro = _("Romania")
    ru = _("Russian Federation")
    rw = _("Rwanda")
    lc = _("Saint Lucia")
    ws = _("Samoa")
    sm = _("San Marino")
    st = _("Sao Tome and Principe")
    sa = _("Saudi Arabia")
    sn = _("Senegal")
    rs = _("Serbia")
    sc = _("Seychelles")
    sl = _("Sierra Leone")
    sg = _("Singapore")
    sk = _("Slovakia")
    si = _("Slovenia")
    sb = _("Solomon Islands")
    so = _("Somalia")
    za = _("South Africa")
    kr = _("South Korea")
    es = _("Spain")
    lk = _("Sri Lanka")
    kn = _("St Kitts and Nevis")
    vc = _("St Vincent and the Grenadines")
    sd = _("Sudan")
    sr = _("Suriname")
    sz = _("Swaziland")
    se = _("Sweden")
    ch = _("Switzerland")
    sy = _("Syria")
    tj = _("Tajikistan")
    tw = _("Taiwan")
    tz = _("Tanzania")
    th = _("Thailand")
    tl = _("Timor Leste")
    tg = _("Togo")
    to = _("Tonga")
    tt = _("Trinidad and Tobago")
    tn = _("Tunisia")
    tr = _("Turkey")
    tm = _("Turkmenistan")
    tc = _("Turks and Caicos Islands")
    tv = _("Tuvalu")
    ug = _("Uganda")
    ua = _("Ukraine")
    ae = _("United Arab Emirates")
    gb = _("United Kingdom")
    us = _("United States of America")
    uy = _("Uruguay")
    uz = _("Uzbekistan")
    vu = _("Vanuatu")
    va = _("Vatican City")
    ve = _("Venezuela")
    vn = _("Viet Nam")
    vg = _("Virgin Islands British")
    vi = _("Virgin Islands US")
    eh = _("Western Sahara")
    ye = _("Yemen")
    zm = _("Zambia")
    zw = _("Zimbabwe")

    UNITED_KINGDOM = Choices.Group(300)
    """Parts of United Kingdom."""

    england = _("England")
    northern_ireland = _("Northern Ireland")
    wales = _("Wales")
    scotland = _("Scotland")

    UNRECOGNIZED_STATES = Choices.Group(600)
    """De facto countries that are not globally recognized."""
    cy_northern = _("Northern Cyprus")
    palestine = _("Palestine")
    somaliland = _("Somaliland")

    UNIONS = Choices.Group(900)
    """Commonly referred unions and associations."""

    african_union = _("African Union")
    arab_league = _("Arab League")
    association_of_southeast_asian_nations = \
            _("Association of Southeast Asian Nations")
    caricom = _("Caricom")
    commonwealth_of_independent_states = \
            _("Commonwealth of Independent States")
    commonwealth_of_nations = _("Commonwealth of Nations")
    european_union = _("European Union")
    islamic_conference = _("Islamic Conference")
    nato = _("NATO")
    olimpic_movement = _("Olimpic Movement")
    opec = _("OPEC")
    red_cross = _("Red Cross")
    united_nations = _("United Nations")


def _language_lookup_getter(overrides, getter):
    def impl(cls, name, getter=lambda choice: choice, fallback=unset):
        """
        If the given `name` has ``-`` characters, they are converted to
        ``_`` for lookup purposes. If no language is found, a more generic
        language lookup is tried (e.g. for ``"pl-pl"`` also ``"pl"`` will be
        attempted) before raising ValueError or returning the fallback value.
        """
        name = name.replace("-", "_")
        name_generic = name.split("_")[0]
        if name_generic == name:
            name_generic = None
        for k, v in cls.__dict__.items():
            exact_match = k == name
            generic_match = name_generic and k == name_generic
            if isinstance(v, ChoicesEntry) and exact_match or generic_match:
                return getter(v)
        if fallback is unset:
            raise ValueError("Nothing found for '{}'.".format(id))
        else:
            return fallback
    function = partial(impl, getter=getter)
    function.__name__ = overrides.__name__
    function.__doc__ = overrides.__doc__ + " " + dedent(impl.__doc__)
    return classmethod(function)


class Language(Choices):
    """Specifies a broad set of languages. Uses a superset of values found in
    Django and Firefox sources."""

    _ = Choices.Choice

    aa = _("Afar")
    ab = _("Abkhazian")
    ae = _("Avestan")
    af = _("Afrikaans")
    ak = _("Akan")
    am = _("Amharic")
    an = _("Aragonese")
    ar = _("Arabic")
    as_ = _("Assamese")
    ast = _("Asturian")
    av = _("Avaric")
    ay = _("Aymara")
    az = _("Azerbaijani")
    ba = _("Bashkir")
    be = _("Belarusian")
    bg = _("Bulgarian")
    bh = _("Bihari")
    bi = _("Bislama")
    bm = _("Bambara")
    bn = _("Bengali")
    bo = _("Tibetan")
    br = _("Breton")
    bs = _("Bosnian")
    ca = _("Catalan")
    ce = _("Chechen")
    ch = _("Chamorro")
    co = _("Corsican")
    cr = _("Cree")
    cs = _("Czech")
    cu = _("Church Slavic")
    cv = _("Chuvash")
    cy = _("Welsh")
    da = _("Danish")
    de = _("German")
    dv = _("Divehi")
    dz = _("Dzongkha")
    ee = _("Ewe")
    el = _("Greek")
    en = _("English")
    en_gb = _("British English")
    en_us = _("American English")
    eo = _("Esperanto")
    es = _("Spanish")
    es_ar = _("Argentinian Spanish")
    et = _("Estonian")
    eu = _("Basque")
    fa = _("Persian")
    ff = _("Fulah")
    fi = _("Finnish")
    fj = _("Fijian")
    fo = _("Faroese")
    fr = _("French")
    fur = _("Friulian")
    fy = _("Frisian")
    ga = _("Irish")
    gd = _("Scots Gaelic")
    gl = _("Galician")
    gn = _("Guarani")
    gu = _("Gujarati")
    gv = _("Manx")
    ha = _("Hausa")
    he = _("Hebrew")
    hi = _("Hindi")
    ho = _("Hiri Motu")
    hr = _("Croatian")
    hsb = _("Upper Sorbian")
    ht = _("Haitian")
    hu = _("Hungarian")
    hy = _("Armenian")
    hz = _("Herero")
    ia = _("Interlingua")
    id = _("Indonesian")
    ie = _("Interlingue")
    ig = _("Igbo")
    ii = _("Sichuan Yi")
    ik = _("Inupiaq")
    io = _("Ido")
    is_ = _("Icelandic")
    it = _("Italian")
    iu = _("Inuktitut")
    ja = _("Japanese")
    jv = _("Javanese")
    ka = _("Georgian")
    kg = _("Kongo")
    ki = _("Kikuyu")
    kj = _("Kuanyama")
    kk = _("Kazakh")
    kl = _("Greenlandic")
    km = _("Khmer")
    kn = _("Kannada")
    ko = _("Korean")
    kok = _("Konkani")
    kr = _("Kanuri")
    ks = _("Kashmiri")
    ku = _("Kurdish")
    kv = _("Komi")
    kw = _("Cornish")
    ky = _("Kirghiz")
    la = _("Latin")
    lb = _("Luxembourgish")
    lg = _("Ganda")
    li = _("Limburgan")
    ln = _("Lingala")
    lo = _("Lao")
    lt = _("Lithuanian")
    lu = _("Luba-Katanga")
    lv = _("Latvian")
    mg = _("Malagasy")
    mh = _("Marshallese")
    mi = _("Maori")
    mk = _("Macedonian")
    ml = _("Malayalam")
    mn = _("Mongolian")
    mr = _("Marathi")
    ms = _("Malay")
    mt = _("Maltese")
    my = _("Burmese")
    na = _("Nauru")
    nb = _("Norwegian Bokm\u00e5l")
    nd = _("Ndebele, North")
    ne = _("Nepali")
    ng = _("Ndonga")
    nl = _("Dutch")
    nn = _("Norwegian Nynorsk")
    no = _("Norwegian")
    nr = _("Ndebele, South")
    nso = _("Sotho, Northern")
    nv = _("Navajo")
    ny = _("Chichewa")
    oc = _("Occitan")
    oj = _("Ojibwa")
    om = _("Oromo")
    or_ = _("Oriya")
    os = _("Ossetian")
    pa = _("Punjabi")
    pi = _("Pali")
    pl = _("Polish")
    ps = _("Pashto")
    pt = _("Portuguese")
    pt_br = _("Brazilian Portugese")
    qu = _("Quechua")
    rm = _("Rhaeto-Romanic")
    rn = _("Kirundi")
    ro = _("Romanian")
    ru = _("Russian")
    rw = _("Kinyarwanda")
    sa = _("Sanskrit")
    sc = _("Sardinian")
    sd = _("Sindhi")
    se = _("Northern Sami")
    sg = _("Sango")
    si = _("Singhalese")
    sk = _("Slovak")
    sl = _("Slovenian")
    sm = _("Samoan")
    sn = _("Shona")
    so = _("Somali")
    sq = _("Albanian")
    sr = _("Serbian")
    sr_latn = _("Serbian Latin")
    ss = _("Siswati")
    st = _("Sotho, Southern")
    su = _("Sundanese")
    sv = _("Swedish")
    sw = _("Swahili")
    ta = _("Tamil")
    te = _("Telugu")
    tg = _("Tajik")
    th = _("Thai")
    ti = _("Tigrinya")
    tig = _("Tigre")
    tk = _("Turkmen")
    tl = _("Tagalog")
    tlh = _("Klingon")
    tn = _("Tswana")
    to = _("Tonga")
    tr = _("Turkish")
    ts = _("Tsonga")
    tt = _("Tatar")
    tw = _("Twi")
    ty = _("Tahitian")
    ug = _("Uighur")
    uk = _("Ukrainian")
    ur = _("Urdu")
    uz = _("Uzbek")
    ve = _("Venda")
    vi = _("Vietnamese")
    vo = _("Volap\u00fck")
    wa = _("Walloon")
    wen = _("Sorbian")
    wo = _("Wolof")
    xh = _("Xhosa")
    yi = _("Yiddish")
    yo = _("Yoruba")
    za = _("Zhuang")
    zh = _("Chinese")
    zh_cn = _("Simplified Chinese")
    zh_tw = _("Traditional Chinese")
    zu = _("Zulu")

    from_name = _language_lookup_getter(overrides=Choices.from_name,
        getter=lambda choice: choice)

    id_from_name = _language_lookup_getter(overrides=Choices.id_from_name,
        getter=lambda choice: choice.id)

    desc_from_name = _language_lookup_getter(overrides=Choices.desc_from_name,
        getter=lambda choice: choice.desc)

    raw_from_name = _language_lookup_getter(overrides=Choices.raw_from_name,
        getter=lambda choice: choice.raw)

    # deprecated compatibility layer for code using lck.django.choices < 0.8
    # to be removed at 1.0
    FromName = from_name
    IDFromName = id_from_name
    DescFromName = desc_from_name
    RawFromName = raw_from_name


class Gender(Choices):
    _ = Choices.Choice

    female = _("female")
    male = _("male")
    unspecified = _("unspecified")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from setuptools import setup, find_packages

reload(sys)
sys.setdefaultencoding('utf8')

ld_file = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
try:
    long_description = ld_file.read()
finally:
    ld_file.close()
# We let it die a horrible tracebacking death if reading the file fails.
# We couldn't sensibly recover anyway: we need the long description.

setup (
    name = 'dj.choices',
    version = '0.8.6',
    author = 'Łukasz Langa',
    author_email = 'lukasz@langa.pl',
    description = "An enum implementation for Django forms and models.",
    long_description = long_description,
    url = 'https://github.com/ambv/dj.choices/',
    keywords = 'django dj extra contrib choices enum enumeration',
    platforms = ['any'],
    license = 'MIT',
    package_dir = {'': 'src'},
    packages = find_packages('src'),
    include_package_data = True,
    zip_safe = False, # if only because of the readme file
    namespace_packages = ['dj'],
    install_requires = [
        'django',
    ],

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
    )
